#include "butterflies_populations.h"
#include "inline_overlays.h"
#include "butterflies_main.h"
#include "modules/module_header.h"
#include "actions/cats_actions.h"
#include "butterflies_generations.h"
#include "inline.h"
#include "inline_butterflies.h"


// only run at the start of the year
void bf_area_generation_update(struct cats_grid *grid, struct cats_thread_info *ts)
{
        struct cats_configuration *conf = ts->conf;
        struct conf_data_butterflies *module_conf = CATS_MODULE_DATA;
        int module_id = CATS_MODULE_ID;
        struct grid_data_butterflies *data = grid->grid_modules[module_id].module_data;

        struct cats_vital_rate *rate = &module_conf->butterfly_generations;

        const cats_dt_coord start_row = ts->area.start_row;
        const cats_dt_coord end_row = ts->area.end_row;
        const cats_dt_coord start_col = ts->area.start_col;
        const cats_dt_coord end_col = ts->area.end_col;

        struct statistics *stats = &ts->stats[grid->id];
        const cats_dt_rates ot = grid->param.OT;

        const int64_t egg_cells_id = module_conf->stat_ids[BF_CELLS_WITH_EGGS];
        const int64_t egg_cells_removed_id = module_conf->stat_ids[BF_CELLS_WITH_EGGS];
        int64_t cells_with_eggs = 0;
        int64_t cells_with_eggs_removed = 0;

        for (cats_dt_coord row = start_row; row < end_row; row++) {
                for (cats_dt_coord col = start_col; col < end_col; col++) {

                        float eggs = butterflies_get_eggs(data, row, col);
                        if (get_suitability(grid, row, col) < ot)
                        {
                                if (eggs > 0) cells_with_eggs_removed += 1;
                                butterflies_set_eggs(data, row, col, 0.0f);
                                butterflies_set_generations(data, row, col, 0.0f);
                                set_population_ignore_cc(grid, row, col, 0);

                                continue;
                        }

                        if (cell_excluded_by_overlay(conf, row, col)) {
                                if (eggs > 0) cells_with_eggs_removed += 1;
                                butterflies_set_eggs(data, row, col, 0.0f);
                                butterflies_set_generations(data, row, col, 0.0f);
                                set_population_ignore_cc(grid, row, col, 0);
                                continue;
                        }

                        int32_t cell_info = butterflies_get_info_layer(data, row, col);
                        if ( ! (cell_info & BF_CELL_VALID_DISPERSAL_TARGET)){
                                if (eggs > 0) cells_with_eggs_removed += 1;
                                butterflies_set_eggs(data, row, col, 0.0f);
                                butterflies_set_generations(data, row, col, 0.0f);
                                set_population_ignore_cc(grid, row, col, 0);
                                continue;
                        }

                        cats_dt_rates gen = calculate_rate(rate, 0, conf->param, grid, row, col, NULL);
                        butterflies_set_generations(data, row, col, (float) gen);
                        if (eggs > 0) cells_with_eggs += 1;
                }
        }

        stats->custom_stats[egg_cells_id] += cells_with_eggs;
        stats->custom_stats[egg_cells_removed_id] += cells_with_eggs_removed;


}